require "io"
require "os"
require "lfs" -- http://keplerproject.github.com/luafilesystem/manual.html

-- module internal data
local internaldata = {}


local pwd = function() return lfs.currentdir() end

local fsload = function(filename)
	local rfsexists, handler = pcall(loadfile, filename)
	if rfsexists and handler then
		return handler
	end

	return nil
end

local getattrmode = function(name)
	return lfs.attributes(name, "mode")
end
local isDirectory = function(name)
	local mode = getattrmode(name)
	return mode == "directory"
end
local isFile = function(name)
	local mode = getattrmode(name)
	return mode == "file"
end
local exists = function(name)
	local mode = getattrmode(name)
	return mode and true or false
end

local enumerate = function(dir)
	-- Note: lfs is for lua filesystem not love filesystem
	local files = {}
	for file in lfs.dir(dir) do
		if file ~= "." and file ~= ".." then
			files:insert(file)
		end
	end
end

local mkdir = function(name)
	--TODO: is creation accept absolute path ?
	--TODO: only create inside SaveDirectory
	return false
end

internaldata.identity = "" -- "" or nil ?
internaldata.source = "NotSet" -- used by setSource
internaldata.XDG_DATA_HOME = os.getenv("XDG_DATA_HOME")
internaldata.HOME = os.getenv("HOME")

if internaldata.XDG_DATA_HOME then
	internaldata.AppdataDirectory = internaldata.XDG_DATA_HOME
	--internaldata.SaveDirectory = string.format("%s/%s", internaldata.XDG_DATA_HOME, "/love/"
elseif internaldata.HOME then
	internaldata.AppdataDirectory = internaldata.HOME
	--internaldata.SaveDirectory = (internaldata.HOME, "./local/share/love/"):format("%s/%s")
else
	error("No such XDG_DATA_HOME or HOME environnement")
end
internaldata.UserDirectory = internaldata.HOME
internaldata.WorkingDirectory = pwd()

assert(love, "LOVE")
assert(love.filesystem, "LOVE.filesystem")

love.filesystem.enumerate           = enumerate
love.filesystem.exists              = exists
love.filesystem.getAppdataDirectory = function() return internaldata.AppdataDirectory end
love.filesystem.getLastModified     = TODO
love.filesystem.getSaveDirectory    = function() return internaldata.SaveDirectory end
love.filesystem.getUserDirectory    = function() return internaldata.UserDirectory end
love.filesystem.getWorkingDirectory = function() return internaldata.WorkingDirectory end
love.filesystem.init                = function() end
love.filesystem.isDirectory         = isDirectory
love.filesystem.isFile              = isFile
love.filesystem.lines               = TODO
love.filesystem.load                = fsload
love.filesystem.mkdir               = mkdir
love.filesystem.newFile             = TODO
love.filesystem.newFileData         = TODO
love.filesystem.read                = TODO
love.filesystem.remove              = TODO
love.filesystem.setIdentity         = function(name)
	internaldata.identity = name end
	--FIXME: need to update AppdataDirectory and/or SaveDirectory
love.filesystem.setSource           = function(source) internaldata.source = source end
love.filesystem.write               = TODO


-- custom function (not present in v0.7.1)
love.filesystem.getIdentity         = function() return internaldata.identity end

-- register the internal data into main table
getInternalDataTable().filesystem = internaldata

